# Generated by Django 3.2.15 on 2022-09-08 17:06

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='HelpTextString',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model_label', models.CharField(editable=False, max_length=255)),
                ('identifier', models.CharField(editable=False, max_length=255)),
                ('text', models.TextField(blank=True)),
            ],
            options={
                'ordering': ('model_label', 'identifier'),
            },
        ),
        migrations.AddIndex(
            model_name='helptextstring',
            index=models.Index(fields=['model_label', 'identifier'], name='help_text_identifier_idx'),
        ),
        migrations.AddConstraint(
            model_name='helptextstring',
            constraint=models.UniqueConstraint(fields=('model_label', 'identifier'), name='unique_help_text_identifier'),
        ),
    ]



from django.utils.html import format_html
from django.templatetags.static import static

try:
    from wagtail import hooks
except ImportError:
    from wagtail.core import hooks

from wagtail.contrib.modeladmin.options import (
    ModelAdmin, modeladmin_register)
from .models import HelpTextString, populate_help_text_strings


_has_populated_records = False


class HelpTextAdmin(ModelAdmin):
    model = HelpTextString
    menu_label = "Help text"
    menu_icon = "help"
    add_to_settings_menu = True
    list_display = ('identifier', 'model_label', 'text')
    search_fields = ('identifier', 'model_label', 'text')
    list_filter = ('model_label', )

    def get_queryset(self, request):
        # On first call to any ModelAdmin view that retrieves the queryset,
        # ensure that all HelpText definitions in the code have been turned into
        # HelpTextString records in the database
        global _has_populated_records
        if not _has_populated_records:
            populate_help_text_strings()
            _has_populated_records = True

        return super().get_queryset(request)


modeladmin_register(HelpTextAdmin)


@hooks.register("insert_global_admin_css")
def editable_help_css():
    return format_html('<link rel="stylesheet" href="{}">', static('wagtail_editable_help/wagtail_editable_help.css'))



"""
Called by CircleCI when the nightly build fails.

This reports an error to the #nightly-build-failures Slack channel.
"""
import os
import requests

if 'SLACK_WEBHOOK_URL' in os.environ:
    print("Reporting to #nightly-build-failures slack channel")
    response = requests.post(os.environ['SLACK_WEBHOOK_URL'], json={
        "text": "A Nightly build failed. See " + os.environ['CIRCLE_BUILD_URL'],
    })

    print("Slack responded with:", response)

else:
    print("Unable to report to #nightly-build-failures slack channel because SLACK_WEBHOOK_URL is not set")



from asgiref.local import Local


_local_data = Local()


class EditableHelpMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if hasattr(request, 'user'):
            _local_data.user = request.user

        response = self.get_response(request)

        try:
            del _local_data.user
        except AttributeError:
            pass

        return response


def get_active_user():
    return getattr(_local_data, 'user', None)



from django.db import models
from django.utils.deconstruct import deconstructible
from django.utils.html import format_html
from django.utils.translation import gettext as _
from telepath import StringAdapter
from wagtail.admin.admin_url_finder import AdminURLFinder

try:
    from wagtail.telepath import register
except ImportError:
    from wagtail.core.telepath import register

from .middleware import get_active_user


class HelpTextString(models.Model):
    model_label = models.CharField(max_length=255, editable=False)
    identifier = models.CharField(max_length=255, editable=False)
    text = models.TextField(blank=True)

    def __str__(self):
        return "%s - %s" % (self.identifier, self.model_label)

    def __repr__(self):
        return "<HelpText: %s %s=%r>" % (self.model_label, self.identifier, self.text)

    class Meta:
        ordering = ("model_label", "identifier",)
        constraints = [
            models.UniqueConstraint(fields=["model_label", "identifier"], name="unique_help_text_identifier"),
        ]
        indexes = [
            models.Index(fields=["model_label", "identifier"], name="help_text_identifier_idx")
        ]


_help_text_objects = set()


@deconstructible
class HelpText:
    def __init__(self, model_label, identifier, default=""):
        self.model_label = model_label
        self.identifier = identifier
        self.default = default
        _help_text_objects.add(self)

    def __str__(self):
        str, created = HelpTextString.objects.get_or_create(
            model_label=self.model_label, identifier=self.identifier,
            defaults={'text': self.default}
        )
        user = get_active_user()
        if user:
            finder = AdminURLFinder(user)
            edit_url = finder.get_edit_url(str)
        if user and edit_url:
            if str.text:
                return format_html('{} <a href="{}" class="edit-help-text">{}</a>', str.text, edit_url, _("Edit"))
            else:
                return format_html('<a href="{}" class="add-help-text">{}</a>', edit_url, _("Add help text"))
        else:
            return str.text

    def __hash__(self):
        return hash((self.model_label, self.identifier))

    def __eq__(self, other):
        return (
            isinstance(other, HelpText)
            and other.model_label == self.model_label
            and other.identifier == self.identifier
        )


def populate_help_text_strings():
    """
    Ensure that a HelpTextString record exists in the database for all HelpText
    definitions that have been encountered during module loading
    """
    for item in _help_text_objects:
        str(item)


class HelpTextAdapter(StringAdapter):
    def build_node(self, obj, context):
        return super().build_node(str(obj), context)


register(HelpTextAdapter(), HelpText)
